#ifndef lint
static char vcid[] ="$Id$";
#endif

/*
    This file contains routines for the NONBLOCKING startup of remote
    processes.

    This is broken down into
    SUT_IRemStart - Remote, nonblocking start
    SUT_RemTestAny - Test for any ready/failures
    SUT_RemTestSome - Test for all that have started/failed
    SUT_RemWaitAny  - Wait (with timeout) for any
    SUT_RemWaitAll  - Wait (with timeout) for all
 */

#include "sut.h" 

/*@
    SUT_IRemStart - Starts a process on a remote node

    Notes:
    The process is not available for use until it has been waited for,
    using SUT_RemTestxxx or SUT_RemWaitxxx.

    The exact interface to this routine (argument string) is still
    under discussion.

    Currently, information about the initialized connection is placed into
    the ith connection entry in jobs.  The state is SUT_STARTED.  fd_start
    has the descriptor set that will be used by the test/wait routines to
    check for a successful connection.  (perhaps this should be more
    abstract, or part of jobs?)
@*/
int SUT_IRemStart( jobs, i, fd_start, pgm, Argv )
SUT_Jobs *jobs;
int      i;
fd_set   *fd_start;
char     *pgm;
char     **Argv;
{
int err;
SUT_Connection *conn = jobs + i;

if (err = SUT_Fork_with_pipes( &conn->pid, &conn->readfd, &conn->writefd ))
    return err;

if (conn->pid == 0) {
    /* Child */
    SUT_execvp( pgm, Argv );
    return SUT_EXEC_FAILED;
    }

conn->state = SUT_STARTED;
FD_SET(conn->readfd, fd_start );
return SUT_SUCCESS;
}

int SUT_RemTestSome( jobs, nready, nfail )
SUT_Jobs *jobs;
int      *nready;
int      *nfail;
{
struct timeval timer;
fd_set         rd_ready;
timer.tv_sec  = 0;
timer.tv_usec = 0;

nready = select( fd + 1, &rd_ready, (fd_set *)0, (fd_set *)0, &timer );

if (nready > 0) {
    /* Look for EOF or handshake from each available connection */
    for (i=0; i<jobs->nconn; i++) {
    	conn = jobs->conn + i;
    	if (conn->state == SUT_STARTED && FD_ISSET(conn->rdfd,&rd_ready) {
	    /* ... Handshake goes here ... */
    	    }
        }
    }
return SUT_SUCCESS;
}
